import pathlib
from urllib.request import urlretrieve

from django.apps import AppConfig
from django.conf import settings

from .templatetags import DEFAULT_VERSION


class MermaidConfig(AppConfig):
    name = "django_mermaid"

    def ready(self):
        """Download mermaid.js from CDN if not already present"""
        version = getattr(settings, "MERMAID_VERSION", DEFAULT_VERSION)
        cdn = "https://cdnjs.cloudflare.com/ajax/libs/mermaid/%s/mermaid.min.js" % version
        current_dir = pathlib.Path(__file__).parent
        static_dir = current_dir / "static"
        mermaid_dir = static_dir / "mermaid" / version
        mermaid_js = mermaid_dir / "mermaid.js"
        if not mermaid_js.exists() or \
               mermaid_js.stat().st_size == 0:
            mermaid_dir.mkdir(parents=True, exist_ok=True)
            urlretrieve(cdn, str(mermaid_js))



import json

from django import template
from django.conf import settings
from django.templatetags.static import static
from django.utils.safestring import mark_safe

from . import DEFAULT_THEME
from . import DEFAULT_VERSION

register = template.Library()


@register.simple_tag
@mark_safe
def mermaid(diagram=None, theme=None):
    """Render a mermaid diagram.

    :param diagram: The mermaid diagram definition
    :param theme: The mermaid theme to use (default, forest, dark, neutral, base). See https://mermaid.js.org/config/theming.
    """

    version = getattr(settings, "MERMAID_VERSION", DEFAULT_VERSION)
    theme = theme or getattr(settings, "MERMAID_THEME", DEFAULT_THEME)
    theme_variables = getattr(settings, "MERMAID_THEME_VARIABLES", {}) if theme == "base" else {}

    mermaid_uri = static("mermaid/%s/mermaid.js" % version)
    html = "<div class=\"mermaid\">%s</div><script src=\"%s\"></script>" % (diagram or "", mermaid_uri)
    init_properties = {"startOnLoad": True, "theme": theme, "themeVariables": theme_variables}
    return html + "<script>mermaid.initialize(%s);</script>" % json.dumps(init_properties)
